%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% FEASIBLE PERIODIC MOTION PRIMITIVES FOR CHOREOGRAPHED 
% QUADROCOPTER FLIGHT (submitted to ACC 2010)
% 
% **Creates feasibility plots for data files created by feasibilityData.m** 
% (Section V, VI) 
%
% Sergei Lupashin & Angela Schoellig, Sept 2010 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


% The function plotFeasibility(filename,showData,outFile,plotTitle) 
% loads the file 'filename.mat' obtained from feasibilityData.m. It creates 
% a feasibility plot and displays experimental data if showData=1.  
% The figure is saved in 'outfile' in the folder figures using 'plotTitle' 
% as title. 

function plotFeasibility(filename,showData,outFile,plotTitle)

% load data file obtained from feasibilityData.m
load(filename);

% defines maximum amplitude and frequency (in rad) displayed in the plots
P=[];
P.ampMax = 1.5;
P.omegaMax = 20;

% reduce the frequency vector WVec (obtained from loaded data file) accordingly 
WVec = WVec(WVec<P.omegaMax);
n = length(WVec);
P.omegaMax = WVec(end);

% feasibility boundary (largest feasible amplitude for a given frequency) 
% with respect to a single motor's thrust limit
low = bounds(:,1)';
low = low(1:n);

% feasibility boundary (largest feasible amplitude for a given frequency) 
% with respect to the collective thrust limit 
high = collA;
high = high(1:n);


%% create the plots
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

figure()
hold on;

% define font
fn = 'Times New Roman';

% location of jbfill.m which is used below to fill a region
% between two lines with a color
addpath('util');

% get position of plot in centimeter
set(gcf,'units','centimeters');
pos = get(gcf,'position');

% labels
xl = xlabel('Frequency (rad/s)','FontName',fn)
yl = ylabel('Amplitude (m)','FontName',fn)
title(plotTitle, 'FontName', fn)

if(showData)
    % plots data from filename.mat together with experimental data
    
    % plots the feasibility regions defined by filename.mat, see
    % util/jbfill.m for details
    p = jbfill(WVec,ones(1,n)*P.ampMax,low,[.9 .9 .9]);
    p=jbfill(WVec,high,low,[.95 .95 .95]);
    hold on;
    p=plot(WVec,low,'k','LineWidth',1,'color',[.7 .7 .7]);
    p=plot(WVec, high,'k','LineWidth',1,'color',[.7 .7 .7]);
    
    set(gcf,'position',[pos(1:2),8.5,7]);
    set(gca, 'position', [0.12 0.14 .81 .80])
    set(xl, 'Position', [10 -0.2/P.ampMax 1]);
    text(11,P.ampMax*.7,'INFEASIBLE','FontName',fn);
    hold on;
    %includes experimental data in plot (see below)
    p=doPlot();  
    
else
    % plots feasibility data from filename.mat 
    
    % plots the feasibility regions defined by filename.mat, see
    % util/jbfill.m for details
    p=jbfill(WVec,ones(1,n)*P.ampMax,low,[.7 .7 .7]);
    p=jbfill(WVec,high,low,[.85 .85 .85]);
    text(11,P.ampMax*.7,'INFEASIBLE','FontName',fn,'color',[1 1 1]);
    hold on;
    p=plot(WVec,low,'k','LineWidth',1);
    p=plot(WVec, high,'k','LineWidth',1);
    
    set(gcf,'position',[pos(1:2),8.5,5]);    
    set(gca, 'position', [0.12 0.18 .81 .73])
    set(xl, 'Position', [10 -0.25/P.ampMax 1]);
end

text(.8,P.ampMax*.10,'FEASIBLE','FontName',fn);



% more adjustments to the figure
set(p, 'LineWidth', 1);
set(gca,'FontName',fn);


xlim([0 P.omegaMax]);
ylim([0 P.ampMax])
set(gca, 'Box', 'on');
pos = get(xl, 'Position');

set(yl, 'Rotation', 90)
set(yl, 'Position', [-1.7 P.ampMax/2 1])

set(gcf,'PaperPositionMode','auto');
%exportfig(gcf, 'side2side.ai','color','cmyk','Resolution',600);
%plot2svg('moo.svg',gcf,1);

% save figure
print('-depsc2',outFile);

end

% Function plotting our experimental results (the function is very much
% adapted to the data we collected)
function p = doPlot

sw=[];

% goes through frequencies (tested on the real experiment)
for hz=.7:.1:2
    hold on;
    s = sprintf('%.1f',hz);
    d = importdata(['data/periodic_data_20100927_sidetoside_' s 'hz_5p.txt']);
    %plot(d(:,2),d(:,6),'color',(1-((hz-.7)/(2-.7)*.8+.2))*[1 1 1]);
    
    % linear interpolation for amplitudes that were not explicitly tested
    % tested amplitudes (for the given frequency hz) are contained in
    % d(:,2)
    tmp = interp1(d(:,2),d(:,:),d(1,2):.001:d(end,2));
    % checks if motor saturation (stored in d(:,6)) is larger than 1
    % percent
    this_sw = tmp(tmp(:,6)>.01,:);
    % lowest amplitude where saturation occurs with 1 percent 
    this_sw = this_sw(1,2);

    % checks if motor saturation (stored in d(:,6)) is larger than 5
    % percent
    this_sw2 = tmp(tmp(:,6)>.05,:);
    % lowest amplitude where saturation occurs with 5 percent
    this_sw2 = this_sw2(1,2);
    
    sw = [sw; hz this_sw this_sw2];
end

% include experimentally obtained feasibility bounds into plot
p=plot(sw(:,1)*2*pi,sw(:,2),'rs','LineWidth',2,'MarkerFaceColor',[1 .7 .7],...
    'MarkerEdgeColor',[.8 0 0]);

end